<?php
// $Id: kdbphp.php 6292 2010-01-19 15:44:58Z s_voec01 $
// Author: Sebastian Voecking <sebastian.voecking@uni-muenster.de>

//ADEI::RequireClass("KDBFileServer", true);

function vectori_to_array($v)
{
    $a = array();

    if (is_resource($v)) {
        for ($i = 0; $i < vectori_size($v); $i++) {
            $a[] = vectori_get($v, $i);
        }
    }

    return $a;
}

function vectori_from_array(array $a)
{
    $size = count($a);
    $r = new_vectori($size);
    for ($i = 0; $i < $size; $i++) {
        vectori_set($r, $i, $a[$i]);
    }

    return $r;
}

function vectord_to_array($v)
{
    $a = array();

    if (is_resource($v)) {
        for ($i = 0; $i < vectord_size($v); $i++) {
            $a[] = vectord_get($v, $i);
        }
    }

    return $a;
}

function vectord_from_array(array $a)
{
    $size = count($a);
    $r = new_vectord($size);
    for ($i = 0; $i < $size; $i++) {
        vectord_set($r, $i, $a[$i]);
    }

    return $r;
}

class KDBPhp
{
    public $_cPtr = 0;

    function __construct()
    {
        if (!extension_loaded("kdbphp")) {
            throw new ADEIException("KDBPhp extension is not loaded!");
        }
        try {
            $this->_cPtr = KDBPhp_Self();
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function GetRun($number)
    {
        try {
            return KDBPhp_GetRun($this->_cPtr, $number);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function GetRunDescription(KDBRunIdentifier $run)
    {
        try {
            return KDBPhp_GetRunDescription($this->_cPtr, $run->self);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function ListRuns($from, $to)
    {
        try {
            return KDBPhp_ListRuns($this->_cPtr, $from, $to);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }


    //function GetFileServer()
    //{
    //    try {
    //        return new KDBFileServer(gFileServer_get());
    //    }
    //    catch (Exception $e) {
    //        throw new ADEIException($e->getMessage());
    //    }
    //}

    function PrintFile(KDBFileIdentifier $file)
    {
        if (!$this->FileExists($file)) {
            $filename = $file->GetFilename();
            if ($file->GetRun()->IsValid()) {
                $runname = $file->GetRun()->GetName();
                $message =
                    "File \"$filename\" does not exist for subrun $runname.";
            }
            else {
                $message = "File \"$filename\" does not exist.";
            }
            throw new ADEIException($message);
        }

        try {
            $path = $this->GetAbsolutePath($file);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }

        readfile($path);
    }

    function GetAbsolutePath(KDBFileIdentifier $file)
    {
        try {
            return KDBPhp_GetAbsolutePath($this->_cPtr, $file->self);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function FileExists(KDBFileIdentifier $file)
    {
        try {
            return KDBPhp_FileExists($this->_cPtr, $file->self);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function HaveRuns($from, $to)
    {
        try {
            return KDBPhp_HaveRuns($this->_cPtr, $from, $to);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function HaveCountrates($from, $to)
    {
        try {
            return KDBPhp_HaveCountrates($this->_cPtr, $from, $to);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }
}

?>
