<?php
// $Id: katrinreader.php 6292 2010-01-19 15:44:58Z s_voec01 $
// Author: Sebastian Voecking <sebastian.voecking@uni-muenster.de>

ADEI::RequireClass("KDBPhp", true);

abstract class KATRINData implements Iterator
{
    public $_cPtr = NULL;
    private $iterator = NULL;

    abstract protected function create($kdb, $from, $to, $v);

    function __construct(KATRINReader &$reader, OPTIONS &$opts, array &$items,
        INTERVAL &$ivl, $resample)
    {
        if ($resample) {
            print $resample . "\n";
        }
        $kdb = $reader->kdb;
        $from = $ivl->GetWindowStart();
        $to = $ivl->GetWindowEnd();
        $pixels = $items;

        try {
            $v = vectori_from_array(array_keys($pixels));
            $this->_cPtr = $this->create($kdb->_cPtr, $from, $to, $v);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function rewind()
    {
        try {
            KDBDataIter_Rewind($this->_cPtr);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function current()
    {
        try {
            $r = KDBDataIter_Current($this->_cPtr);
            $a = vectord_to_array($r);
            return $a;
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function next()
    {
        try {
            return KDBDataIter_Next($this->_cPtr);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function valid()
    {
        try {
            return KDBDataIter_Valid($this->_cPtr);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function key()
    {
        try {
            return KDBDataIter_Key($this->_cPtr);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }
}

class KATRINCountrates extends KATRINData
{
    protected function create($kdb, $from, $to, $items)
    {
        return new_KDBCountrateIter($kdb, $from, $to, $items);
    }
}

class KATRINRuns extends KATRINData
{
    protected function create($kdb, $from, $to, $items)
    {
        return new_KDBRunIter($kdb, $from, $to, $items);
    }
}

class KATRINReader extends READER
{
    public $kdb = NULL;

    function __construct(&$props)
    {
        parent::__construct($props);

        $this->kdb = new KDBPhp();

        $this->items = array();
        $this->items['countrates'] = array();
        $this->items['countrates'][] = array('name' => "Total (IPE)");
        $this->items['countrates'][] = array('name' => "Total (UW)");
        for ($i = 0; $i < 64; $i++) {
            $name = "Pixel " . $i . " (IPE)";
            $this->items['countrates'][] = array('name' => $name);
        }
        for ($i = 0; $i < 64; $i++) {
            $name = "Pixel " . $i . " (UW)";
            $this->items['countrates'][] = array('name' => $name);
        }

        $this->items['runs'] = array();
        $this->items['runs'][] = array('name' => "Run numbers");
        $this->items['runs'][] = array('name' => "Run numbers (no subruns)");
        $this->items['runs'][] = array('name' => "Activity");

        $this->groups = array(
            'countrates' => array('name' => "Pixel countrates"),
            'runs' => array('name' => "Runs")
        );
    }

    function GetGroupInfo(LOGGROUP $grp = NULL, $flags = 0)
    {
        $groups = array();

        foreach ($this->groups as $gid => &$group) {
            if ($grp && strcmp($grp->gid, $gid)) continue;

            if ($group['name']) $name = $group['name'];

            if (!$name || ($flags & REQUEST::NEED_INFO)) {
                if ($grp) {
                    $grtest = $grp;
                    $opts = $this->opts;
                }
                else {
                    $ginfo = array("db_group" => $gid);
                    $grtest = $this->CreateGroup($ginfo);
                    $opts = $this->req->GetGroupOptions($grtest);
                }

                if (!$name) {
                    $name = $opts->Get('name', $gid);
                }
            }

            $groups[$gid] = array('gid' => $gid, 'name' => $name);

            if ($flags & REQUEST::NEED_INFO) {
                $limit = $opts->GetDateLimit(0, time());
                $groups[$gid]['first'] = $limit[0];
                $groups[$gid]['last'] = $limit[1];

                if ($flags & REQUEST::NEED_COUNT) {
                    $period = $opts->Get('period');
                    $groups[$gid]['records'] =
                        ($groups[$gid]['last'] - $groups[$gid]['first']) /
                        $period;
                }

                if ($flags & REQUEST::NEED_ITEMINFO) {
                    $groups[$gid]['items'] = $this->GetItemList($grtest);
                }
            }
        }

        if ($grp && !$groups) {
            $error = translate("Invalid group (%s) is specified", $grp->gid);
            throw new ADEIException($error);
        }

        return $grp ? $groups[$grp->gid] : $groups;
    }

    function GetItemList(LOGGROUP $grp = NULL, MASK $mask = NULL, $flags = 0)
    {
        if ($flags & REQUEST::ONLY_AXISINFO) {
            if (!$this->req->GetGroupOptions($grp, "axis")) return array();
        }

        $grp = $this->CheckGroup($grp);
        if (!$mask) $mask = $this->CreateMask($grp);

        $items = array();

        $opts = $this->req->GetGroupOptions($grp);
        $items = $opts->Get('items', $this->items[$grp->gid]);

        $res = array();

        for ($i = 0; isset($items[$i]); $i++) {
            if (!$mask->Check($i)) continue;

            $item = array("id" => $i, "name" => $items[$i]['name']);

            if ($items[$i]['uid']) {
                $item['uid'] = $items[$i]['uid'];
            }

            if ($flags & REQUEST::NEED_AXISINFO) {
                if ($grp->GetGroupID() == "countrates") {
                    $item["axis"] = "counts/sec";
                }
                elseif ($grp->GetGroupID() == "runs") {
                    $item["axis"] = "number";
                }
            }

            $res[] = $item;
        }

        return $res;
    }

    function GetMaskList(LOGGROUP $grp = NULL, $flags = 0)
    {
        //print "KATRINReader::GetMaskList";

        $grp = $this->CheckGroup($grp);

        $masks = array();

        if ($grp->GetGroupID() == "runs") {
        }
        else if ($grp->GetGroupID() == "countrates") {
            $mask = array();
            $mask["id"] = "total";
            $mask["name"] = "Total";
            if ($flags & REQUEST::NEED_INFO) {
                $mask["mask"] = "0,1";
            }
            $masks[] = $mask;

            $mask = array();
            $mask["id"] = "ipe";
            $mask["name"] = "Pixels (IPE)";
            if ($flags & REQUEST::NEED_INFO) {
                $items = array();
                for ($i = 2; $i < 66; $i++) {
                    $items[] = $i;
                }
                $mask["mask"] = implode(",", $items);
            }
            $masks[] = $mask;

            $mask = array();
            $mask["id"] = "uw";
            $mask["name"] = "Pixels (UW)";
            if ($flags & REQUEST::NEED_INFO) {
                $items = array();
                for ($i = 67; $i < 130; $i++) {
                    $items[] = $i;
                }
                $mask["mask"] = implode(",", $items);
            }
            $masks[] = $mask;
        }

        return array_merge(parent::GetMaskList($grp, $flags), $masks);
    }

    function GetRawData(LOGGROUP $grp = NULL, $from = 0, $to = 0,
        DATAFilter $filter = NULL, &$filter_data = NULL)
    {
        $grp = $this->CheckGroup($grp);

        $ivl = $this->CreateInterval($grp);
        $ivl->Limit($from, $to);

        if ($filter) {
            $mask = $filter->GetItemMask();
            $resample = $filter->GetSamplingRate();
            $limit = $filter->GetVectorsLimit();
            if ($limit) $ivl->SetItemLimit($limit);

            if (isset($filter_data)) {
                if ($mask) $filter_data['masked'] = true;
                if ($resample) $filter_data['resampled'] = true;
                if ($limit) $filter_data['limited'] = true;
            }
        }
        else {
            $mask = NULL;
            $resample = 0;
            $limit = 0;
        }

        $opts = $this->req->GetGroupOptions($grp);

        $items = $opts->Get('items', $this->items[$grp->gid]);
        if ($mask && is_array($ids = $mask->GetIDs())) {
            $tmp = array();
            foreach ($ids as $id) {
                if ($id >= sizeof($items)) {
                    $error =
                        translate("Invalid item mask is supplied. " .
                        "The ID:%d refers non-existing item.", $id);
                    throw new ADEIException($error);
                }
                array_push($tmp, $items[$id]);
            }
            $items = $tmp;
        }

        if ($grp->GetGroupID() == "countrates") {
            return new KATRINCountrates($this,
                $this->req->GetGroupOptions($grp), $items, $ivl, $resample);
        }
        else if ($grp->GetGroupID() == "runs") {
            return new KATRINRuns($this, $this->req->GetGroupOptions($grp),
                $items, $ivl, $resample);
        }
        else {
            return NULL;
        }
    }

    function HaveData(LOGGROUP $grp = NULL, $from = 0, $to = 0)
    {
        if (is_null($grp) || $grp->GetGroupID() == "runs") {
            return $this->kdb->HaveRuns($from, $to);
        }
        else if ($grp->GetGroupID() == "countrates") {
            return $this->kdb->HaveCountrates($from, $to);
        }
        else {
            return FALSE;
        }
    }
}

?>
