<?php
// $Id: katrin.php 6273 2010-01-18 15:44:27Z s_voec01 $
// Author: Sebastian Voecking <sebastian.voecking@uni-muenster.de>

//require("../adei.php");
//require($ADEI_ROOTDIR . "/setups/katrin/classes/kdbphp.php");

ADEI::RequireClass("KDBFileIdentifier", true);
ADEI::RequireClass("KDBPhp", true);
ADEI::RequireClass("KDBRunIdentifier", true);
ADEI::RequireClass("INTERVAL");

function PrintXml($xml)
{
    header("Content-type: text/xml");
    header("Cache-Control: no-cache, must-revalidate");
    header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");

    print $xml;
}

function GetContentType($filename)
{
    if (substr($filename, -4) == ".png") {
        return "image/png";
    }
    else if (substr($filename, -4) == ".txt") {
        return "text/plain";
    }
    else if (substr($filename, -4) == ".xml") {
        return "application/xml";
    }
    else {
        return "application/octet-stream";
    }
}

function PrintFile($path, $download)
{
    if (!file_exists($path))
        throw new ADEIException("File $path does not exist");

    $mimetype = GetContentType($path);
    $basename = basename($path);

    header("Content-type: $mimetype");
    if ($download)
        header("Content-disposition: attachment; filename=\"$basename\"");
    readfile($path);
}

function PrintError($error)
{
    $xml = "<?xml version=\"1.0\"?>\n<result>\n";
    $xml .= " <Error>$error</Error>\n</result>";

    PrintXml($xml);
}

try {
    $kdb = new KDBPhp();

    switch ($_GET['target']) {
        case 'runs':
            $interval = new INTERVAL($_GET);
            $start = $interval->GetWindowStart();
            $end = $interval->GetWindowEnd();

            PrintXml($kdb->ListRuns($start, $end));
            break;

        case 'info':
            if (isset($_GET['run'])) {
                PrintXml($kdb->GetRun($_GET['run']));
            }
            else
                throw new ADEIException("No run specified");
            break;

        case 'rundesc':
            if (!isset($_GET['run']))
                throw new ADEIException("No run specified");
            $identifier = new KDBRunIdentifier($_GET['run']);
            PrintXml($kdb->GetRunDescription($identifier));
            break;

        case 'file':
            $file = new KDBFileIdentifier();
            if (isset($_GET['run'])) {
                $identifier = new KDBRunIdentifier($_GET['run']);
                $file->SetRun($identifier);
            }

            if (isset($_GET['filename'])) {
                $file->SetFilename($_GET['filename']);
            }

            if (isset($_GET['type'])) {
                $type = $_GET['type'];
                if ($type == "orca") {
                    $file->SetFileType(kTypeOrca);
                }
                if ($type == "orcaroot") {
                    $file->SetFileType(kTypeOrcaRoot);
                }
                if ($type == "crun") {
                    $file->SetFileType(kTypeCrun);
                }
                if ($type == "header") {
                    $file->SetFileType(kTypeHeader);
                }
                if ($type == "plot") {
                    $file->SetFileType(kTypePlot);
                }
            }

            PrintFile($kdb->GetAbsolutePath($file), $download);
            break;

        default:
            if (isset($_GET['target'])) {
                $target = $_GET['target'];
                $message = "Unknown target ($target) is specified";
                throw new ADEIException($message);
            }
            else
                throw new ADEIException("The target is not specified");
    }
}
catch (ADEIException $e) {
    PrintError($e->getMessage());
}
?>
